define(['box2d'], function(Box2D) {

	// global classnames from Box2d namespace
	var b2Vec2 = Box2D.Common.Math.b2Vec2,
		b2BodyDef = Box2D.Dynamics.b2BodyDef,
		b2Body = Box2D.Dynamics.b2Body,
		b2FixtureDef = Box2D.Dynamics.b2FixtureDef,
		b2World = Box2D.Dynamics.b2World,
		b2PolygonShape = Box2D.Collision.Shapes.b2PolygonShape,
		b2CircleShape = Box2D.Collision.Shapes.b2CircleShape;

	// x, y gravity : here it means 10m/s2 almost earth gravity
	var gravity = new b2Vec2(0, 10);
	
	// fixture shared between the ball and the floor
	var fixDef = new b2FixtureDef();
	fixDef.density = 1.0;
	fixDef.friction = 0.2;
	fixDef.restitution = 0.3;
	
	// private fields
	var width=800, 
		height=600;
	
	/**
	 * Constructor of the scene
	 */
	function Scene() {
		// scaling factor : 1 unit in box2d equals 30 pixels
		this.SCALE = 30;
		// creation of the box2d world
		this.world = new b2World(gravity, true);
		// add a ball
		this.ball = this.addBall(width/2/this.SCALE, 20/this.SCALE);
		// add a floor
		this.floor = this.addFloor();
	}
	
	/**
	 * Add a ball to the scene
	 * @param x {int} the x coordinate
	 * @param y {int} the y coordinate
	 */
	Scene.prototype.addBall = function (x, y) {
		// create a new body in the world
		var bodyDef = new b2BodyDef();
		bodyDef.type = b2Body.b2_dynamicBody;
		// position
		bodyDef.position.x = x;
		bodyDef.position.y = y;
		// create the fixture's shape associated to the ball
		fixDef.shape = new b2CircleShape(40/this.SCALE);
		// add the body to the b2web world
		return this.world.CreateBody(bodyDef).CreateFixture(fixDef);
	};
	
	
	/**
	 * Add a floor to the scene
	 */
	Scene.prototype.addFloor = function() {
		// creation of the floor body
		var bodyDef = new b2BodyDef();
		// make it static
		bodyDef.type = b2Body.b2_staticBody;
		// set the position of the body
		bodyDef.position.x = width/2/this.SCALE;
		bodyDef.position.y = (height-1)/this.SCALE;
		// creation of the shape governed by the fixture
		fixDef.shape = new b2PolygonShape();
		fixDef.shape.SetAsBox(width/2/this.SCALE, 1/this.SCALE);
		// add the ground body and its fixture to the world		
		return this.world.CreateBody(bodyDef).CreateFixture(fixDef);
	};
	
	// return the scene class
	return Scene;
});